# WS 320x240 display example
from machine import Pin,SPI,PWM
import framebuf
import utime
import os
import gc
import math
import random

def report():
    gc.enable()
    print("Allocated: " +str(gc.mem_alloc()))
    print("Free: "+str(gc.mem_free()))
    print("----------------")
    gc.collect()
    print("Allocated: " +str(gc.mem_alloc()))
    print("Free: "+str(gc.mem_free()))
    print("================\n")
    
print("Imports & Reporting")
report()

BL = 13
DC = 8
RST = 12
MOSI = 11
SCK = 10
CS = 9

class LCD_1inch3(framebuf.FrameBuffer): # For 320x240 display
    def __init__(self):
        self.width = 320
        self.height = 240
        
        self.cs = Pin(CS,Pin.OUT)
        self.rst = Pin(RST,Pin.OUT)
        
        self.cs(1)
        self.spi = SPI(1)
        self.spi = SPI(1,1000_000)
        self.spi = SPI(1,100000_000,polarity=0, phase=0,sck=Pin(SCK),mosi=Pin(MOSI),miso=None)
        self.dc = Pin(DC,Pin.OUT)
        self.dc(1)
        self.buffer = bytearray(self.height * self.width * 2)
        super().__init__(self.buffer, self.width, self.height, framebuf.RGB565)
        self.init_display()
        
        self.RED   =   0x07E0
        self.GREEN =   0x001f
        self.BLUE  =   0xf800
        self.WHITE =   0xffff
        self.BALCK =   0x0000
        
    def write_cmd(self, cmd):
        self.cs(1)
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
        self.cs(1)

    def write_data(self, buf):
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(bytearray([buf]))
        self.cs(1)

    def init_display(self):
        """Initialize display"""  
        self.rst(1)
        self.rst(0)
        self.rst(1)
        
        self.write_cmd(0x36)
        self.write_data(0x70)

        self.write_cmd(0x3A) 
        self.write_data(0x05)

        self.write_cmd(0xB2)
        self.write_data(0x0C)
        self.write_data(0x0C)
        self.write_data(0x00)
        self.write_data(0x33)
        self.write_data(0x33)

        self.write_cmd(0xB7)
        self.write_data(0x35) 

        self.write_cmd(0xBB)
        self.write_data(0x19)

        self.write_cmd(0xC0)
        self.write_data(0x2C)

        self.write_cmd(0xC2)
        self.write_data(0x01)

        self.write_cmd(0xC3)
        self.write_data(0x12)   

        self.write_cmd(0xC4)
        self.write_data(0x20)

        self.write_cmd(0xC6)
        self.write_data(0x0F) 

        self.write_cmd(0xD0)
        self.write_data(0xA4)
        self.write_data(0xA1)

        self.write_cmd(0xE0)
        self.write_data(0xD0)
        self.write_data(0x04)
        self.write_data(0x0D)
        self.write_data(0x11)
        self.write_data(0x13)
        self.write_data(0x2B)
        self.write_data(0x3F)
        self.write_data(0x54)
        self.write_data(0x4C)
        self.write_data(0x18)
        self.write_data(0x0D)
        self.write_data(0x0B)
        self.write_data(0x1F)
        self.write_data(0x23)

        self.write_cmd(0xE1)
        self.write_data(0xD0)
        self.write_data(0x04)
        self.write_data(0x0C)
        self.write_data(0x11)
        self.write_data(0x13)
        self.write_data(0x2C)
        self.write_data(0x3F)
        self.write_data(0x44)
        self.write_data(0x51)
        self.write_data(0x2F)
        self.write_data(0x1F)
        self.write_data(0x1F)
        self.write_data(0x20)
        self.write_data(0x23)
        
        self.write_cmd(0x21)

        self.write_cmd(0x11)

        self.write_cmd(0x29)

    def show(self):
        self.write_cmd(0x2A)
        self.write_data(0x00)
        self.write_data(0x00)
        self.write_data(0x01)
        self.write_data(0x3f)
        
        self.write_cmd(0x2B)
        self.write_data(0x00)
        self.write_data(0x00)
        self.write_data(0x00)
        self.write_data(0xEF)
        
        self.write_cmd(0x2C)
        
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(self.buffer)
        self.cs(1)
        

# Colour Mixing Routine
def colour(R,G,B): # Compact method!
    mix1 = ((R&0xF8)*256) + ((G&0xFC)*8) + ((B&0xF8)>>3)
    return  (mix1 & 0xFF) *256  + int((mix1 & 0xFF00) /256) # low nibble first

def ring(cx,cy,r,cc):   # Draws a circle - with centre (x,y), radius, colour 
    for angle in range(91):  # 0 to 90 degrees in 2s
        y3=int(r*math.sin(math.radians(angle)))
        x3=int(r*math.cos(math.radians(angle)))
        LCD.pixel(cx-x3,cy+y3,cc)  # 4 quadrants
        LCD.pixel(cx-x3,cy-y3,cc)
        LCD.pixel(cx+x3,cy+y3,cc)
        LCD.pixel(cx+x3,cy-y3,cc)
        
# ==== Main ====  
pwm = PWM(Pin(BL))
pwm.freq(1000)
pwm.duty_u16(32768)#max 65535

LCD = LCD_1inch3()
print("Initialize Screen")
report()
LCD.fill(LCD.WHITE)
LCD.show()

LCD.fill(0)

LCD.rect(0,0,320,240,colour(0,0,255)) # Blue Frame
LCD.text("WaveShare",44,10,colour(255,0,0))
LCD.text('Pico Display 2.0"',10,24,colour(255,255,0))
LCD.text("320x240 SPI",38,37,colour(0,255,0))
LCD.text("Tony Goodhew",30,48,colour(150,150,150))
LCD.show()

LCD.pixel(0,0,0xFFFF)     # Left Top - OK
LCD.pixel(0,239,0xFFFF)   # Left Bottom - OK
LCD.pixel(319,0,0xFFFF)   # Right Top - OK
LCD.pixel(319,239,0xFFFF) # Right Bottom - OK
LCD.show()
utime.sleep(4)
LCD.fill(0)
LCD.text("This is RED text",10,10,colour(255,0,0))
LCD.hline(30,30,200,colour(0,0,255))
LCD.vline(30,40,150,colour(0,255,0))
LCD.line(35,35,100,100,colour(255,255,0))
LCD.line(35,35,200,100,colour(255,255,0))
LCD.rect(105,105,100,80,colour(255,0,255))
LCD.fill_rect(110,110,90,70,colour(0,255,255))
LCD.text("Tony",130,130,0)
LCD.text("Goodhew",130,140,0)
LCD.show()
print("After GFX demo")
report()

# Set up buttons
key0 = Pin(15,Pin.IN,Pin.PULL_UP)
key1 = Pin(17,Pin.IN,Pin.PULL_UP)
key2 = Pin(2 ,Pin.IN,Pin.PULL_UP)
key3= Pin(3 ,Pin.IN,Pin.PULL_UP)
utime.sleep(2)
white = 0xFFFF
LCD.text("Buffer: "+str(320*240*2) +" bytes",50,220,white)

LCD.show()
utime.sleep(2)
red=colour(255,0,0)
green = colour(0,255,0)
xx = 260
yy = 15
for i in range(4):
    LCD.text(str(i),xx,yy+10,white)
    yy =yy + 60
LCD.fill_rect(50,220,220,8,0)
LCD.text("Press all the buttons to stop",30,220,white)
LCD.show()
running = True
while running:
    t = 0
    if key0.value() == 0:
        LCD.fill_rect(275,15,30,30,red)
        t = t + 1
    else :
        LCD.fill_rect(275,15,30,30,green) # Block
        LCD.rect(275,15,30,30,red)        # Edge
               
    if(key1.value() == 0):
        LCD.fill_rect(275,75,30,30,red)
        t = t + 2
    else :
        LCD.fill_rect(275,75,30,30,green)
        LCD.rect(275,75,30,30,red)
                
    if(key2.value() == 0):
        LCD.fill_rect(275,135,30,30,red)
        t = t + 4
    else :
        LCD.fill_rect(275,135,30,30,green)
        LCD.rect(275,135,30,30,red)
        
    if(key3.value() == 0):
        LCD.fill_rect(275,195,30,30,red)
        t = t + 8
    else :
        LCD.fill_rect(275,195,30,30,green)
        LCD.rect(275,195,30,30,red)
        
    LCD.show()    
    if t == 15: # Halt?
        running = False
               
print("After buttons")
report()
LCD.fill(0)
LCD.text("Halted",134,120,white)
for r in range(100,110,1):
    ring(160,120,r,colour(255,255,0))
    ring(160,120,r+10,colour(0,0,255))
    ring(160,120,r-10,colour(0,255,0))
LCD.show()
utime.sleep(3)
print("After Rings #1 - Halted")
report()
# Dice
LCD.fill(0)
LCD.show()
b = colour(0,0,255)
r = colour(255,0,0)
y = colour(255,255,0)
colwidth = 20
totals = [0,0,0,0,0,0,0,0,0,0,0,0,0]
for i in range(2,13):
    LCD.text(str(i),15+i*colwidth,225,y)
LCD.hline(50,220,230,b)
LCD.vline(50,40,180,b)
for i in range(1,191,10):
    LCD.hline(40,39+i,10,b)
for i in range(1,191,50):
    LCD.hline(35,69+i,245,b)
for i in range(0,151,50):
    LCD.text(str(i),8,217-i,b)
LCD.text("1000 throws of 2 dice",75,190,r)
for i in range(1000):
    LCD.fill_rect(20,19,220,10,0)
    left = random.randint(1,6)
    LCD.text(str(left),20,20,r)
    right = random.randint(1,6)    
    LCD.text(str(right),50,20,r)
    t = left + right
    LCD.text(str(t),80,20,y)
    LCD.text(str(i+1),200,20,r)
    totals[t] = totals[t] + 1
    LCD.hline(11+t*colwidth,220-totals[t],colwidth-1,y)
    if i == 800:
        LCD.text("1000 throws of 2 dice",75,190,b)
    LCD.show()
    
utime.sleep(5)
print("After bar graph")
report()

LCD.fill(0)
LCD.text("Finished",130,120,white)
for r in range(100,110,1):
    ring(160,120,r,colour(255,255,0))
    ring(160,120,r+10,colour(0,0,255))
    ring(160,120,r-10,colour(0,255,0))
LCD.show()
utime.sleep(5)
LCD.fill(0)
LCD.show()
print("At end of program")
report()
